from pathlib import Path

from aiopath import AsyncPath
from acb import Action, action_registry, base_path


def register_actions() -> None:
    _actions_path = Path(base_path / "actions")
    if not _actions_path.exists():
        _actions_path = Path(__file__).parent
    _pkg = _actions_path.parent.name
    _actions = {
        a.stem: a
        for a in _actions_path.iterdir()
        if a.is_file() and not a.name.startswith("_")
    }
    _pkg_path = _actions_path.parent
    for action_name, path in _actions.items():
        _action = next(
            (a for a in action_registry.get() if a.name == action_name), None
        )
        if _action:
            del _action
        action_registry.get().append(Action(name=action_name, path=AsyncPath(path)))


register_actions()



import typing as t

from hishel import (
    AsyncCacheClient,  # type: ignore
    RedisStorage,  # type: ignore
)
from httpx import Response as HttpxResponse
from redis.asyncio import Redis as AsyncRedis
from acb.depends import depends
from ._base import RequestsBase, RequestsBaseSettings


class RequestsSettings(RequestsBaseSettings): ...


class Requests(RequestsBase):
    storage: RedisStorage  # type: ignore

    async def get(self, url: str, timeout: int = 5) -> HttpxResponse:
        async with AsyncCacheClient() as client:
            return await client.get(url, timeout=timeout)

    async def post(
        self, url: str, data: dict[str, t.Any], timeout: int = 5
    ) -> HttpxResponse:
        async with AsyncCacheClient() as client:
            return await client.post(url, data=data, timeout=timeout)

    async def put(
        self, url: str, data: dict[str, t.Any], timeout: int = 5
    ) -> HttpxResponse:
        async with AsyncCacheClient() as client:
            return await client.put(url, data=data, timeout=timeout)

    async def delete(self, url: str, timeout: int = 5) -> HttpxResponse:
        async with AsyncCacheClient(storage=self.storage) as client:
            return await client.delete(url, timeout=timeout)

    @depends.inject
    async def init(self) -> None:  # type: ignore
        self.storage = RedisStorage(
            client=AsyncRedis(
                host=self.config.cache.host.get_secret_value(),
                port=self.config.cache.port,
                db=self.config.requests.cache_db,
            ),
            ttl=self.config.requests.cache_ttl,
        )


depends.set(Requests)



from contextvars import ContextVar
from pathlib import Path

from aiopath import AsyncPath
from pydantic import BaseModel

base_path: AsyncPath = AsyncPath(Path.cwd())
tmp_path = base_path / "tmp"


class Action(BaseModel, arbitrary_types_allowed=True):
    name: str
    pkg: str = "acb"
    module: str = ""
    path: AsyncPath = AsyncPath(__file__) / "actions"


action_registry: ContextVar[list[Action]] = ContextVar("action_registry", default=[])


class Adapter(BaseModel, arbitrary_types_allowed=True):
    name: str
    category: str
    required: bool = False
    enabled: bool = False
    installed: bool = False
    pkg: str = "acb"
    module: str = ""
    path: AsyncPath = AsyncPath(Path(__file__) / "adapters")

    def __str__(self) -> str:
        return self.__repr__()


adapter_registry: ContextVar[list[Adapter]] = ContextVar("adapter_registry", default=[])



import typing as t
from contextlib import suppress
from secrets import compare_digest
from warnings import catch_warnings, filterwarnings

from google.api_core.exceptions import AlreadyExists, PermissionDenied
from google.auth import default
from google.auth.transport.requests import AuthorizedSession, Request
from google.cloud.secretmanager_v1 import (
    AccessSecretVersionRequest,
    AddSecretVersionRequest,
    CreateSecretRequest,
    DeleteSecretRequest,
    ListSecretsRequest,
    SecretManagerServiceAsyncClient,
)
from acb.adapters import import_adapter
from acb.config import app_name, project
from acb.depends import depends
from ._base import SecretBase, SecretBaseSettings

Logger = import_adapter()


class SecretSettings(SecretBaseSettings): ...


class Secret(SecretBase):
    logger: Logger = depends()  # type: ignore
    project: str = ""
    parent: str = ""
    prefix: str = ""
    client: t.Optional[SecretManagerServiceAsyncClient] = None
    authed_session: t.Optional[AuthorizedSession] = None
    creds: t.Optional[t.Any] = None

    def extract_secret_name(self, secret_path: str) -> t.Any:
        return secret_path.split("/")[-1].removeprefix(self.prefix)

    async def get_access_token(self) -> t.Any:
        self.creds.refresh(Request())
        self.logger.debug(f"Secrets access token:\n{self.creds.token}")
        return self.creds.token

    async def verify_access_token(self, token: str) -> t.Any:
        verified = compare_digest(self.creds.token, token)
        self.logger.debug(f"Secrets access token verified - {verified}")
        return verified

    async def list(self, adapter: str) -> t.Any:
        request = ListSecretsRequest(
            parent=self.parent, filter=f"{self.prefix}{adapter}_"
        )
        try:
            client_secrets = await self.client.list_secrets(request=request)
        except PermissionDenied:
            raise SystemExit(
                "\n ERROR:  'project' id in 'settings/app.yml' is invalid or not set!\n"
            )
        client_secrets = [
            self.extract_secret_name(secret.name) async for secret in client_secrets
        ]
        return client_secrets

    async def get(self, name: str) -> t.Any:
        path = f"projects/{self.project}/secrets/{name}/versions/latest"
        request = AccessSecretVersionRequest(name=path)
        version = await self.client.access_secret_version(request=request)
        payload = version.payload.data.decode()
        self.logger.info(f"Fetched secret - {name}")
        return payload

    async def create(self, name: str, value: str) -> t.NoReturn:
        with suppress(AlreadyExists):
            request = CreateSecretRequest(
                parent=self.parent,
                secret_id=name,
                secret={"replication": {"automatic": {}}},
            )
            version = await self.client.create_secret(request)
            request = AddSecretVersionRequest(
                parent=version.name,
                payload={"data": value.encode()},
            )
            await self.client.add_secret_version(request)
            self.logger.debug(f"Created secret - {name}")

    async def update(self, name: str, value: str) -> t.NoReturn:
        secret = self.client.secret_path(self.project, name)
        request = AddSecretVersionRequest(
            parent=secret,
            payload={"data": value.encode()},
        )
        await self.client.add_secret_version(request)
        self.logger.debug(f"Updated secret - {name}")

    async def delete(self, name: str) -> t.NoReturn:
        secret = self.client.secret_path(self.project, name)
        request = DeleteSecretRequest(name=secret)
        await self.client.delete_secret(request=request)
        self.logger.debug(f"Deleted secret - {secret}")

    async def init(self) -> t.NoReturn:
        self.project = project
        self.parent = f"projects/{project}"
        self.prefix = f"{app_name}_"
        with catch_warnings():
            filterwarnings("ignore", category=Warning)
            creds, _ = default()
        self.creds = creds
        self.client = SecretManagerServiceAsyncClient(
            credentials=self.creds  # type: ignore
        )
        self.authed_session = AuthorizedSession(self.creds)


depends.set(Secret)



import typing as t
from abc import ABC, abstractmethod

from acb.config import Settings


class SecretBaseSettings(Settings):
    requires: t.Optional[list[str]] = ["logger"]


class SecretBase(ABC):
    @abstractmethod
    async def list(self, adapter: str) -> t.NoReturn:
        raise NotImplementedError

    @abstractmethod
    async def create(self, name: str, value: str) -> t.NoReturn:
        raise NotImplementedError

    @abstractmethod
    async def update(self, name: str, value: str) -> t.NoReturn:
        raise NotImplementedError

    @abstractmethod
    async def get(self, name: str) -> t.NoReturn:
        raise NotImplementedError

    @abstractmethod
    async def delete(self, name: str) -> t.NoReturn:
        raise NotImplementedError



import typing as t

from cashews.serialize import Serializer, register_type
from msgspec import msgpack
from acb.depends import depends
from ._base import CacheBase, CacheBaseSettings


class CacheSettings(CacheBaseSettings): ...


class Cache(CacheBase):
    @staticmethod
    async def encoder(value: t.Any, *args: t.Any, **kwargs: t.Any) -> bytes:
        return msgpack.encode(value)

    @staticmethod
    async def decoder(value: bytes, *args: t.Any, **kwargs: t.Any) -> t.Any:
        return msgpack.decode(value)

    async def init(self, *args: t.Any, **kwargs: t.Any) -> t.NoReturn:
        await super().init(
            "mem://",
            prefix=self.config.cache.prefix,
        )
        register_type(Serializer, self.encoder, self.decoder)


depends.set(Cache)



from time import perf_counter

from acb.adapters import import_adapter
from acb.config import Config
from acb.depends import depends
from ._base import AppBase, AppBaseSettings

main_start = perf_counter()

Logger = import_adapter()


class AppSettings(AppBaseSettings): ...


class App(AppBase):
    async def init(self) -> None:
        self.logger.info("Application starting...")

        # put app startup code here

        async def post_startup() -> None:
            if not self.config.deployed:
                from aioconsole import aprint
                from pyfiglet import Figlet

                fig = Figlet(font="slant", width=90, justify="center")
                await aprint(f"\n\n{fig.renderText(self.config.app.name.upper())}\n")
            if not self.config.debug.production and self.config.deployed:
                self.logger.info("Entering production mode...")

        await post_startup()
        main_start_time = perf_counter() - main_start
        self.logger.info(f"App started in {main_start_time} s")

    @depends.inject
    async def main(
        self,
        config: Config = depends(),
        logger: Logger = depends(),  # type: ignore
    ) -> None:
        ...

        # put app main code here


depends.set(App)

app = depends.get(App)
